package org.eclipse.weaver.extension.byADL;

import java.util.Iterator;

import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.TreeIterator;
import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.util.EcoreUtil.Copier;
import org.eclipse.emf.ecore.xmi.impl.EcoreResourceFactoryImpl;
import org.eclipse.m2m.atl.drivers.emf4atl.ASMEMFModelElement;
import org.eclipse.m2m.atl.engine.AtlModelHandler;
import org.eclipse.uml2.uml.PrimitiveType;
import org.eclipse.uml2.uml.Profile;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.uml2.uml.UMLFactory;
import org.eclipse.uml2.uml.internal.impl.ClassImpl;
import org.eclipse.uml2.uml.internal.impl.PackageImpl;
import org.eclipse.uml2.uml.internal.impl.PropertyImpl;
import org.eclipse.uml2.uml.util.UMLUtil;

/**
 * This is the class that produces an Ecore metamodel from a UML profile
 * 
 * @author - Ivano Malavolta (Department of Computer Science)<br>
 * 		   - Universit dell'Aquila (Italy)
 * @since 1.0
 */
public class UMLprofile2Metamodel {

	private EClassifier eBoolean;
	private EClassifier eString;
	private EClassifier eInteger;
	
	public UMLprofile2Metamodel() {
		this.eBoolean = (EClassifier) ((ASMEMFModelElement)AtlModelHandler.getDefault("EMF").getMof().findModelElement("EBoolean")).getObject();
		this.eString = (EClassifier) ((ASMEMFModelElement)AtlModelHandler.getDefault("EMF").getMof().findModelElement("EString")).getObject();
		this.eInteger = (EClassifier) ((ASMEMFModelElement)AtlModelHandler.getDefault("EMF").getMof().findModelElement("EInt")).getObject();
	}

	/**
	 * Replicates all the inherited <code>EStructuralFeatures</code> to the local UML object
	 * 
	 * @param input The <code>Resource</code> containing the UML specification of the profile
	 */
	public void convertUmlInput(Resource input) {
		EcoreResourceFactoryImpl factory = new EcoreResourceFactoryImpl();
		ResourceSet resSet = new ResourceSetImpl();
		resSet.getResourceFactoryRegistry().getExtensionToFactoryMap().put("ecore", factory);
		Resource output = resSet.createResource(input.getURI());
		TreeIterator res = input.getAllContents();
		Object ob;
		Object[] convertedModel;
		int profileIndex;
		try {
			// iterate to find a Profile definition
			while(res.hasNext()) {
				ob = res.next();
				if((ob instanceof Profile)) {
					// convert the profile to an EMF package
					convertedModel = (Object[]) UMLUtil.convertToEcore((PackageImpl) ob, null).toArray();
					profileIndex = this.getProfileIndex(convertedModel, (Profile) ob);
					((EPackage) convertedModel[profileIndex]).setNsURI(input.getURI().toString());
					// for each stereotype of the profile the converter generated an EMF class,
					// so the supertypes references of such class are set to the previously
					// linked metaclasses.
					Copier copier = new Copier();
					// retrieve the UML metamodel from the UML2 plug-in
					EPackage uml = (EPackage) copier.copy(UMLFactory.eINSTANCE.getUMLPackage());
					copier.copyReferences();
					copier.clear();
					// add the UML metamodel package to the Resource
					((EPackage) convertedModel[profileIndex]).getESubpackages().add(uml);
					this.setStereotypesReferencesTypes((EPackage) convertedModel[profileIndex], uml, (Profile) ob);
					this.setStereotypesSuperTypes((EPackage) convertedModel[profileIndex], uml, (Profile) ob);
					//this.setTaggedValuesPrimitiveTypes((EPackage) convertedModel[profileIndex], uml, (Profile) ob);
					this.changeUMLPrimitiveTypesIntoEcore(uml);
					output.getContents().add((EPackage) convertedModel[profileIndex]);
					input.getContents().clear();
					input.getContents().addAll(output.getContents());
				}				
			}
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	/**
	 * change each attribute of the UML package into its Ecore-based version
	 * and removes all the UML primitive types from the UML package.
	 * 
	 * @param umlPack the package containing the UML metamodel
	 */
	private void changeUMLPrimitiveTypesIntoEcore(EPackage umlPack) {
		EList<EClassifier> classes = umlPack.getEClassifiers();
		EList<EAttribute> currents;
		for(int i=0; i<classes.size(); i++) {
			if(classes.get(i) instanceof EClass) {
				currents = ((EClass)classes.get(i)).getEAttributes();
				for(int j=0; j< currents.size(); j++) {
					currents.get(j).setEType(this.getCorrespondingEcorePrimitiveType(currents.get(j)));
				}
			}
		}
	}
	
	private EClassifier getCorrespondingEcorePrimitiveType(EAttribute attribute) {
		
		if(attribute.getEType().getName().equals("Boolean")) {
			return this.eBoolean;
		}
		if(attribute.getEType().getName().equals("Integer")) {
			return this.eInteger;
		}
		if(attribute.getEType().getName().equals("String")) {
			return this.eString;
		}
		if(attribute.getEType().getName().equals("UnlimitedNatural")) {
			return this.eInteger;
		}
		return attribute.getEType();
	}

	/**
	 * For each stereotype of the profile the converter generated an EMF class,
	 * so the supertypes references of such class are set to the previously
	 * linked metaclasses.
	 * @param pack the package obtained from the converted profile
	 * @param umlPack the package containing the UML metamodel
	 * @param profile the profile before the convert
	 */
	private void setStereotypesSuperTypes(EPackage pack, EPackage umlPack, Profile profile) {
		Object[] classes = pack.eContents().toArray();
		Object[] metaclasses;
		Stereotype currentStereotype;
		EClass currentClass;
		for(int j=0; j<classes.length; j++) {
			if(classes[j] instanceof EClass) {
				currentClass = (EClass) classes[j]; 
				currentStereotype = profile.getOwnedStereotype(((EClass) classes[j]).getName());
				metaclasses = currentStereotype.getExtendedMetaclasses().toArray();
				if(currentStereotype.getName().equals(currentClass.getName())) {
					for(int x=0; x<metaclasses.length; x++) {
						// add the extended class to the supertypes of each class of the package
						for(int i=0; i < umlPack.getEClassifiers().size(); i++) {
							if(umlPack.getEClassifiers().get(i).getName().equals(((ClassImpl) metaclasses[x]).getName())) {
								currentClass.getESuperTypes().add((EClass) umlPack.getEClassifiers().get(i));	
							}
						}
						// remove the feature referencing the UML extension element in the stereotype definition
						if(currentClass.getEStructuralFeature("base_" + ((ClassImpl) metaclasses[x]).getName()) != null) {
							currentClass.getEStructuralFeatures().remove(currentClass.getEStructuralFeature("base_" + ((ClassImpl) metaclasses[x]).getName()));
						}
					}
				}
			}
		}
	}

	/**
	 * For each tagged value whose type is primitive, it must reference the metaclass
	 * of the generated uml package.
	 * 
	 * @param pack the package obtained from the converted profile
	 * @param umlPack the package containing the UML metamodel
	 * @param profile the profile before the convert
	 */
	@SuppressWarnings("unused")
	private void setTaggedValuesPrimitiveTypes(EPackage pack, EPackage umlPack, Profile profile) {
		Object[] classes = pack.eContents().toArray();
		Object[] taggedValues;
		Stereotype currentStereotype;
		EClass currentClass;
		for(int j=0; j<classes.length; j++) {
			if(classes[j] instanceof EClass) {
				currentClass = (EClass) classes[j]; 
				currentStereotype = profile.getOwnedStereotype(((EClass) classes[j]).getName());
				taggedValues = currentStereotype.getOwnedAttributes().toArray();
				if(currentStereotype.getName().equals(currentClass.getName())) {
					for(int x=0; x<taggedValues.length; x++) {
						if(((PropertyImpl) taggedValues[x]).getType() instanceof PrimitiveType) {
							for(int i=0; i < umlPack.getEClassifiers().size(); i++) {
								if(umlPack.getEClassifiers().get(i).getName().equals(((PropertyImpl) taggedValues[x]).getType().getName())) {
									currentClass.getEStructuralFeature(((PropertyImpl) taggedValues[x]).getName()).setEType(umlPack.getEClassifiers().get(i));
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * For each tagged value referencing a UML metaclass must reference the metaclass
	 * of the generated uml package,
	 * so the supertypes references of such class are set to the previously
	 * linked metaclasses.
	 * @param pack the package obtained from the converted profile
	 * @param umlPack the package containing the UML metamodel
	 * @param profile the profile before the convert
	 */
	private void setStereotypesReferencesTypes(EPackage pack, EPackage umlPack, Profile profile) {
		Object[] classes = pack.eContents().toArray();
		Object[] references;
		Stereotype currentStereotype;
		EClass currentClass;
		for(int j=0; j<classes.length; j++) {
			if(classes[j] instanceof EClass) {
				currentClass = (EClass) classes[j]; 
				currentStereotype = profile.getOwnedStereotype(((EClass) classes[j]).getName());
				references = currentStereotype.getOwnedAttributes().toArray();
				if(currentStereotype.getName().equals(currentClass.getName())) {
					for(int x=0; x<references.length; x++) {
						if(((PropertyImpl) references[x]).getType() instanceof ClassImpl && !((PropertyImpl) references[x]).getName().startsWith("base_")) {
							for(int i=0; i < umlPack.getEClassifiers().size(); i++) {
								if(umlPack.getEClassifiers().get(i).getName().equals(((PropertyImpl) references[x]).getType().getName())) {
									currentClass.getEStructuralFeature(((PropertyImpl) references[x]).getName()).setEType(umlPack.getEClassifiers().get(i));
								}
							}
						}
					}
				}
			}
		}
	}

	/**
	 * @param currentClass the EMF class
	 * @param umlPack the converted UML metamodel
	 * @return the EMF class with the same name as <code>currentClass</code>.
	 * They are not the same object because <code>currentClass</code> is the result
	 * of a copy.
	 */
	protected EClass getConvertedClass(ClassImpl currentClass, EPackage umlPack) {
		Iterator iterator = umlPack.getEClassifiers().iterator();
		while(iterator.hasNext()) {
			Object current = iterator.next();
			if(currentClass.getName().equals(((ENamedElement) current).getName())) {
				return (EClass) current;
			}
		}
		return null;
	}

	/**
	 * @param convertedModel a List containing some EMF packages.
	 * @return the index of the element of the list containing the UML metamodel.
	 */
	protected int getUmlMetamodelIndex(Object[] convertedModel) {
		for(int i=0; i<convertedModel.length; i++) {
			if(((EPackage)convertedModel[i]).getNsPrefix().equals("uml")
					&& ((EPackage)convertedModel[i]).getName().equals("uml")) {
				return i;
			}
		}
		return 0;
	}

	/**
	 * @param convertedModel a List containing some EMF packages.
	 * @param ob a UML profile
	 * @return the index of the element of the list containing the profile.
	 */
	private int getProfileIndex(Object[] convertedModel, Profile ob) {
		for(int i=0; i<convertedModel.length; i++) {
			if(((EPackage)convertedModel[i]).getName().equals(ob.getName())) {
				return i;
			}
		}
		return 0;
	}
}
