package org.eclipse.weaver.extension.byADL.panel;

import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.emf.ecore.EAttribute;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EStructuralFeature;
import org.eclipse.emf.ecore.impl.DynamicEObjectImpl;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.m2m.atl.drivers.emf4atl.ASMEMFModel;
import org.eclipse.m2m.atl.engine.vm.nativelib.ASMModel;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.weaver.editors.GlobalWeaverEditor;
import org.eclipse.weaver.editors.weaving.GenericWeavingPanel;
import org.eclipse.weaver.extension.byADL.plugin.WeaverByADLExtensionPlugin;
import org.eclipse.weaver.extension.byADL.transformation.TransformationConfig;
import org.eclipse.weaver.extension.byADL.transformation.TransformationsManager;
import org.eclipse.weaver.extension.dually.util.Util;
import org.eclipse.weaver.plugin.WeaverPlugin;
import org.eclipse.weaver.plugin.util.WeaverResourceUtil;


/**
 * ByADL weaving model panel
 * 
 * @author - Ivano Malavolta (Department of Computer Science)<br>
 * 		   - Universit dell'Aquila (Italy)
 * @since 1.0
 */
public class ByADLWeavingPanel extends GenericWeavingPanel {
	private Image composeMetamodels;
	private Image composed2singleGenerator;
	private Image single2composedGenerator;
	private Image expandAllImage;
	private Image collapseAllImage;

	/**
	 * Sets the 'compose Metamodels' button and the action executed when it is activated. 
	 */  
	private void setComposeMetamodelsButton (final TransformationConfig tc){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		composeMetamodels = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/composeMetamodels.gif")).createImage();
		ti.setImage(composeMetamodels);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_composeMetamodels"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					executeTransformation(tc);             		
				}
			}
		});
	}
	
	/**
	 * Sets the 'generate single 2 composed' button and the action executed when it is activated. 
	 */  
	private void setSingle2composedGeneratorButton (final TransformationConfig tc){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		single2composedGenerator = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/single2composedGenerator.gif")).createImage();
		ti.setImage(single2composedGenerator);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_single2composedGenerator"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					executeMigratorGeneratorTransformation(tc);
				}
			}
		});
	}
	
	/**
	 * Sets the 'generate composed 2 single' button and the action executed when it is activated. 
	 */  
	private void setComposed2singleGeneratorButton (final TransformationConfig tc){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		composed2singleGenerator = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/composed2singleGenerator.gif")).createImage();
		ti.setImage(composed2singleGenerator);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_composed2singleGenerator"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					executeMigratorGeneratorTransformation(tc);
				}
			}
		});
	}

	/* (non-Javadoc)
	 * @see org.eclipse.weaver.editors.IWeavingPanel#init(org.eclipse.weaver.editors.GlobalWeaverEditor)
	 */
	public void init(GlobalWeaverEditor editor) {
		this.globalEditor = editor;
	}	

	/* 
	 * This method is overriden to avoid problems derived from the menu item "weaving element" in the generic weaving panel.
	 * It does nothing.
	 * 
	 * (non-Javadoc)
	 * @see org.eclipse.weaver.editors.weaving.GenericWeavingPanel#populateMenuAboutToShow(org.eclipse.jface.action.IMenuManager)
	 */
	@Override
	public void populateMenuAboutToShow(IMenuManager parentMenu) {
	}

	/**
	 * Sets the CollapseAll button and the action executed when it is activated. 
	 */
	private void setCollapseAllButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		collapseAllImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/collapseAll.gif")).createImage();
		ti.setImage(collapseAllImage);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_collapse"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					((TreeViewer)viewer).collapseAll();             		
				}
			}
		});
	} 

	/**
	 * Sets the ExpandAll button and the action executed when it is activated. 
	 */    
	private void setExpandAllButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		expandAllImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/expandAll.gif")).createImage();
		ti.setImage(expandAllImage);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_expand"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					((TreeViewer)viewer).expandAll();             		
				}
			}
		});

	}

	/**
	 * Sets the SearchElement button and the action executed when it is activated. 
	 */        
	private void setSearchElementButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		expandAllImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/search.gif")).createImage();
		ti.setImage(expandAllImage);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_search_element"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){            	             		
					InputDialog msgDialog = new InputDialog(viewer.getControl().getShell(),
							WeaverPlugin.getPlugin().getString("_UI_search_element_title"),
							WeaverPlugin.getPlugin().getString("_UI_search_element_caption"),"",null);             		
					if (msgDialog.open() == org.eclipse.jface.window.Window.OK){
						boolean found = false;
						List list = new ArrayList();
						for (Iterator i = ((Resource)((ResourceSet) getViewer().getInput()).getResources().get(0)).getAllContents(); i.hasNext();){             	    		
							Object obj = i.next();
							if (obj instanceof EObject){             	    				
								EStructuralFeature feature = ((EObject)obj).eClass().getEStructuralFeature("name");
								if (feature != null){//search the Name prop             	    					
									Object name =((EObject)obj).eGet(feature);
									if (name !=null && name instanceof String){
										found = ((String)name).toLowerCase().indexOf(msgDialog.getValue().toLowerCase()) >= 0;
										if (found){
											list.add(obj);
											getViewer().setSelection(new StructuredSelection(obj));
										}        								
									}
								}
							}
						}
						if (list.size()==0){
							MessageDialog aDialog = new MessageDialog(viewer.getControl().getShell(),
									WeaverByADLExtensionPlugin.getPlugin().getString("_UI_element_not_found"),null,
									WeaverByADLExtensionPlugin.getPlugin().getString("_UI_element_not_found"),
									0,
									new String[]{WeaverByADLExtensionPlugin.getPlugin().getString("_UI_confirm_button")},0);
							aDialog.open();						        									
						}        					
					}
				}
			}
		});               

	}	

	/**
	 * Creates a title bar for the pane which includes the view icon and title to the far left
	 */
	protected void createTitleBar() {
		// Only do this once.
		if (titleLabel == null) {
			// Title.
			titleLabel = new CLabel(control, SWT.SHADOW_NONE);
			titleLabel.setAlignment(SWT.LEFT);
			titleLabel.setBackground(null, null);
			updateTitles();
			control.setTopLeft(titleLabel);
			titleLabel.addMouseListener(mouseListener);
		}
		actionBar = new ToolBar(control, SWT.FLAT | SWT.WRAP);
		/*collapse all*/
		setCollapseAllButton();        
		/*expand all*/
		setExpandAllButton();
		/* selects an element*/
		setSearchElementButton();
		/* generate composing transformation */
		this.setComposeMetamodelsButton(TransformationsManager.getCompositionTransformation());
		/* generate single 2 composed transformation */
		this.setSingle2composedGeneratorButton(TransformationsManager.getSingle2ComposedGenerator());
		/* generate composed 2 single transformation */
		this.setComposed2singleGeneratorButton(TransformationsManager.getComposed2SingleGenerator());
		control.setTopCenter(actionBar);
	}
	/**
	 * Executes the transformation described into the TransformationConfig given as parameter.
	 * 
	 * @param tc the configuration containing the informations about the location and other features 
	 * of the transformation to execute.
	 */
	private void executeTransformation(TransformationConfig tc) {
		try {
			GlobalWeaverEditor editor = globalEditor;
			Map models = TransformationsManager.setModelsMap(null, editor.getModelManager(), false, 
					false, null, null);
			this.storeMetamodelsToTransformationsManager();
			// execute the transformation and get the output model
			ASMModel out = TransformationsManager.executeCompositionTransfo(models, editor.getModelManager(), globalEditor.getXmlFile(), tc);
			// save the model as an ecore file
			ASMModel saved = TransformationsManager.saveNewModel(tc, out, editor.getModelManager(), editor.getXmlFile());
			//this.extractTransformation(saved);
			//refreshes the resource and shows confirmation message
			WeaverResourceUtil.refreshResourceFolder(editor.getModelManager().getWeavingModel());
			MessageDialog.openInformation(editor.getEditorSite().getShell(), tc.description, 
					"New metamodel created:"
					+ ((ASMEMFModel)saved).getExtent().getURI().toString());
		} catch (Exception e) {
			e.printStackTrace();
		}		
	}
	
	/**
	 * Executes the transformation described into the TransformationConfig given as parameter.
	 * 
	 * @param tc the configuration containing the informations about the location and other features 
	 * of the transformation to execute.
	 */
	private void executeMigratorGeneratorTransformation(TransformationConfig tc) {
		try {
			GlobalWeaverEditor editor = globalEditor;
			Map models = TransformationsManager.setMigratorGeneratorModelsMap(null, editor.getModelManager(), false, 
					false, null, null);
			this.storeMetamodelsToTransformationsManager();
			// execute the transformation and get the output model
			ASMModel out = TransformationsManager.executeGeneratorsTransfo(models, editor.getModelManager(), globalEditor.getXmlFile(), tc);
			// save the model as an ecore file
			ASMModel saved = TransformationsManager.saveNewMigrator(tc, out, editor.getModelManager(), editor.getXmlFile());
			//this.extractTransformation(saved);
			//refreshes the resource and shows confirmation message
			WeaverResourceUtil.refreshResourceFolder(editor.getModelManager().getWeavingModel());
			MessageDialog.openInformation(editor.getEditorSite().getShell(), tc.description, 
					"New migrator created:"
					+ ((ASMEMFModel)saved).getExtent().getURI().toString());
		} catch (Exception e) {
			e.printStackTrace();
		}		
	}

	private void storeMetamodelsToTransformationsManager() {
		ByADLWovenModelPanel currentPanel;
		for(int i=0; i<this.globalEditor.getWeaverModelPanels().size(); i++) {
			if(this.globalEditor.getWeaverModelPanels().get(i) instanceof ByADLWovenModelPanel) {
				currentPanel = (ByADLWovenModelPanel) this.globalEditor.getWeaverModelPanels().get(i);
				TransformationsManager.addMetamodel((Resource) currentPanel.getViewer().getInput());
			}
		}
	}

	@Override
	public void dispose() {
		super.dispose();
		//		this.viewer = null;
		//		MemoryMXBean mxbean = ManagementFactory.getMemoryMXBean();
		//		System.out.println("heap: " + mxbean.getHeapMemoryUsage());
		//		System.out.println("\tnon-heap: " + mxbean.getNonHeapMemoryUsage());
	}

	@Override
	public void setInput(ResourceSet input) {
		super.setInput(input);
		//this.setDefaultRootName(input);
	}

	/**
	 * @param input
	 * 
	 * If the name of the byADL weaving model root is unset, this method
	 * sets the name of the byADL root element of the weaving model with the name 
	 * of the weaving model file.
	 */
	public void setDefaultRootName(ResourceSet input) {
		try {
			String rootName;
			EObject root = (DynamicEObjectImpl) ((TreeViewer) this.getViewer()).getTree().getItems()[0].getItems()[0].getData();
			EAttribute feature = (EAttribute) root.eClass().getEStructuralFeature("name");
			rootName = (String) root.eGet(feature);
			if(rootName == null || rootName.trim().equals("")) {
				rootName = Util.getFileName((Resource) ((TreeViewer) this.getViewer()).getTree().getItems()[0].getData());
				root.eSet(feature, rootName);
			}
		} catch(Exception e) {
			e.printStackTrace();
			return;
		}
	}


	/* 
	 *    
	 * This method is overriden to avoid problems
	 * derived from the generic weaving panel.
	 * It does nothing.
	 * 
	 * (non-Javadoc)
	 * @see org.eclipse.weaver.editors.weaving.GenericWeavingPanel#createWLinkEndActions(org.eclipse.emf.ecore.EObject)
	 */
	public void createWLinkEndActions(EObject wLinkEnd) {
		try {
			super.createWLinkEndActions(wLinkEnd);
		} catch(Exception e) {
			//e.printStackTrace();
		}
	}
}