package org.eclipse.weaver.extension.byADL.panel;

import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.eclipse.core.resources.IFile;
import org.eclipse.core.resources.IResource;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.Path;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClassifier;
import org.eclipse.emf.ecore.ENamedElement;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.transaction.TransactionalEditingDomain;
import org.eclipse.gmf.ecore.part.EcoreDiagramEditorPlugin;
import org.eclipse.gmf.ecore.part.EcoreDiagramEditorUtil;
import org.eclipse.gmf.ecore.part.EcoreNewDiagramFileWizard;
import org.eclipse.gmf.runtime.emf.core.GMFEditingDomainFactory;
import org.eclipse.jface.dialogs.InputDialog;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.StructuredSelection;
import org.eclipse.jface.viewers.TreeViewer;
import org.eclipse.jface.wizard.Wizard;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.CLabel;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.ToolBar;
import org.eclipse.swt.widgets.ToolItem;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.PartInitException;
import org.eclipse.ui.PlatformUI;
import org.eclipse.ui.dialogs.SaveAsDialog;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.uml2.diagram.profile.part.Messages;
import org.eclipse.uml2.diagram.profile.part.UMLDiagramEditor;
import org.eclipse.uml2.diagram.profile.part.UMLDiagramEditorPlugin;
import org.eclipse.uml2.diagram.profile.part.UMLDiagramEditorUtil;
import org.eclipse.uml2.diagram.profile.part.UMLNewDiagramFileWizard;
import org.eclipse.uml2.uml.internal.resource.UMLResourceImpl;
import org.eclipse.weaver.core.WeaverXMLMetadata;
import org.eclipse.weaver.core.data.WovenModelDescriptor;
import org.eclipse.weaver.editors.weaving.SelectionTreeViewer;
import org.eclipse.weaver.extension.byADL.UMLprofile2Metamodel;
import org.eclipse.weaver.extension.byADL.plugin.WeaverByADLExtensionPlugin;
import org.eclipse.weaver.extension.byADL.providers.ByADLWovenModelLabelProvider;
import org.eclipse.weaver.extension.dually.panel.DUALLyViewerFilter;


import org.eclipse.weaver.extension.dually.util.Util;
import org.eclipse.weaver.extension.panel.DefaultWovenModelPanel;
import org.eclipse.weaver.plugin.util.WeaverUtil;
import org.eclipse.weaver.providers.GenericIdentifierAdapter;
import org.eclipse.weaver.providers.IIdentifierAdapter;
import org.eclipse.weaver.providers.ResourceIdentifierAdapter;
import org.eclipse.weaver.providers.XMIIdentifierAdapter;

/**
 * ByADL woven metamodel panel
 * 
 * @author - Ivano Malavolta (Department of Computer Science)<br>
 * 		   - Universit dell'Aquila (Italy)
 * @since 1.0
 */
public class ByADLWovenModelPanel extends DefaultWovenModelPanel {

	// the object that transforms a UML profile into an Ecore metamodel
	private UMLprofile2Metamodel transformer = new UMLprofile2Metamodel();

	// the images of the toolbar
	private Image graphicalViewImage;
	private Image collapseAllImage;
	private Image expandAllImage;
	// the upper-side toolbar
	private ToolBar actionBar;
	// the current item being added to the toolbar
	private ToolItem ti;
	// XML file (with the extension .amw.prop) containing the metadata of the weaving model being edited
	private WeaverXMLMetadata metadataFile;

	/**
	 * Constructs a view panel for a view part.
	 */
	public ByADLWovenModelPanel() {
		super();
	}

	/**
	 * Sets the button to graphically view the metamodel/profile and the actions executed when it is activated. 
	 */    
	private void setGraphicalViewButton(){
		this.ti = new ToolItem(actionBar, SWT.PUSH, 0);
		graphicalViewImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/graphic.gif")).createImage();
		this.ti.setImage(graphicalViewImage);
		this.ti.setSelection(false);
		this.ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					if(isUML(viewer.getInput())) {
						viewProfile();
					} else {
						viewMetamodel();
					}
				}
			}
		});

	}

	/**
	 * This is the backing method that permits to graphically view the woven UML profile.
	 */
	private void viewProfile() {
		// retrieve the Resource object from the XML metadata file of the weaving model
		Resource profile = Util.getWovenModelFromMetadataFile(this.metadataFile, ((Resource) this.viewer.getInput()).getURI());
		String path = profile.getURI().toPlatformString(true);
		// path contains the path of the UML diagram of the profile
		path += "profile_diagram";
		// find (if there is one) the diagram
		IResource workspaceResource = ResourcesPlugin.getWorkspace().getRoot().findMember(new Path(path));
		// if the ResourcesPlugin finds the diagram, just open the editor (initialized with such profile)
		if (workspaceResource instanceof IFile) {
			IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			try {
				// open the editor
				page.openEditor(new FileEditorInput((IFile) workspaceResource), UMLDiagramEditor.ID);
			} catch (PartInitException e) {
				e.printStackTrace();
			}
			// if the UML diagram does not exist, open the wizard to create one
		} else {
			TransactionalEditingDomain editingDomain = GMFEditingDomainFactory.INSTANCE.createEditingDomain();
			EObject diagramRoot = null;
			try {
				// load the root element of the "semantic" definition of the profile
				diagramRoot = (EObject) profile.getContents().get(0);
			} catch (Exception ex) {
				UMLDiagramEditorPlugin.getInstance().logError("Unable to load resource: " + profile.getURI(), ex);
			}
			if (diagramRoot == null) {
				MessageDialog.openError(this.getControl().getShell(), Messages.UMLInitDiagramFileAction_InitDiagramFileResourceErrorDialogTitle, Messages.UMLInitDiagramFileAction_InitDiagramFileResourceErrorDialogMessage);
				return;
			}
			// open and visualize the wizard
			Wizard wizard = new UMLNewDiagramFileWizard(profile.getURI(), diagramRoot, editingDomain);
			wizard.setWindowTitle(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_graphic_profile"));
			UMLDiagramEditorUtil.runWizard(this.getControl().getShell(), wizard, "InitDiagramFile");
		}
	}

	/**
	 * This is the backing method that permits to graphically view the woven metamodel.
	 */    
	private void viewMetamodel() {
		// retrieve the Resource object from the XML metadata file of the weaving model
		Resource metamodel = Util.getWovenModelFromMetadataFile(this.metadataFile, ((Resource) this.viewer.getInput()).getURI());
		String path = metamodel.getURI().toPlatformString(true);
		// path contains the path of the UML diagram of the metamodel
		path += "_diagram";
		// find (if there is one) the diagram
		IResource workspaceResource = ResourcesPlugin.getWorkspace().getRoot().findMember(new Path(path));
		// if the ResourcesPlugin finds the diagram, just open the editor (initialized with such metamodel)
		if (workspaceResource instanceof IFile) {
			IWorkbenchPage page = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getActivePage();
			try {
				// open the editor
				page.openEditor(new FileEditorInput((IFile) workspaceResource), UMLDiagramEditor.ID);
			} catch (PartInitException e) {
				e.printStackTrace();
			}
			// if the diagram does not exist, open the wizard to create one
		} else {
			TransactionalEditingDomain editingDomain = GMFEditingDomainFactory.INSTANCE.createEditingDomain();
			EObject diagramRoot = null;
			try {
				// load the root element of the "semantic" definition of the metamodel
				diagramRoot = (EObject) metamodel.getContents().get(0);
			} catch (Exception ex) {
				EcoreDiagramEditorPlugin.getInstance().logError("Unable to load resource: " + metamodel.getURI(), ex);
			}
			if (diagramRoot == null) {
				MessageDialog.openError(this.getControl().getShell(), org.eclipse.gmf.ecore.part.Messages.EcoreInitDiagramFileAction_InitDiagramFileResourceErrorDialogTitle, org.eclipse.gmf.ecore.part.Messages.EcoreInitDiagramFileAction_InitDiagramFileResourceErrorDialogMessage);
				return;
			}
			// open and visualize the wizard
			Wizard wizard = new EcoreNewDiagramFileWizard(metamodel.getURI(), diagramRoot, editingDomain);
			wizard.setWindowTitle(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_graphic_metamodel"));
			EcoreDiagramEditorUtil.runWizard(this.getControl().getShell(), wizard, "InitDiagramFile");
		}
	}

	/**
	 * Sets the CollapseAll button and the action executed when it is activated. 
	 */
	private void setCollapseAllButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		collapseAllImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/collapseAll.gif")).createImage();
		ti.setImage(collapseAllImage);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_collapse"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					((TreeViewer)viewer).collapseAll();             		
				}
			}
		});
	} 

	/**
	 * Sets the ExpandAll button and the action executed when it is activated. 
	 */    
	private void setExpandAllButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		expandAllImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/expandAll.gif")).createImage();
		ti.setImage(expandAllImage);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_expand"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){
					((TreeViewer)viewer).expandAll();             		
				}
			}
		});

	}

	/**
	 * Sets the SearchElement button and the action executed when it is activated. 
	 */
	private void setSearchElementButton (){
		ToolItem ti = new ToolItem(actionBar, SWT.PUSH, 0);
		expandAllImage = ImageDescriptor.createFromURL((URL)WeaverByADLExtensionPlugin.getPlugin().getImage("icons/search.gif")).createImage();
		ti.setImage(expandAllImage);
		ti.setSelection(false);
		ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_search_element"));
		ti.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e){
				if(viewer != null){            	             		
					InputDialog msgDialog = new InputDialog(viewer.getControl().getShell(),
							WeaverByADLExtensionPlugin.getPlugin().getString("_UI_search_element_title"),
							WeaverByADLExtensionPlugin.getPlugin().getString("_UI_search_element_caption"),"",null);             		
					if (msgDialog.open() == org.eclipse.jface.window.Window.OK){
						boolean found = false;
						for (Iterator i = WeaverUtil.getAllPackages((Resource)getViewer().getInput()).iterator(); i.hasNext();){             	    		
							EPackage pkg = (EPackage) i.next();
							for (Iterator pkgs = pkg.eAllContents(); pkgs.hasNext();){
								Object obj = pkgs.next();
								if (obj instanceof EClassifier || obj instanceof EPackage){             	    				
									String feature = ((ENamedElement)obj).getName();	
									found = feature.equalsIgnoreCase(msgDialog.getValue());
									if (found){        									
										getViewer().setSelection(new StructuredSelection(obj));
										return; 
									}
								}
							}
						}
						if (!found) {
							MessageDialog aDialog = new MessageDialog(viewer.getControl().getShell(),
									WeaverByADLExtensionPlugin.getPlugin().getString("_UI_element_not_found"),null,
									WeaverByADLExtensionPlugin.getPlugin().getString("_UI_element_not_found"),
									0,new String[]{"OK"},0);
							aDialog.open();						        									
						}
					}
				}
			}
		});               

	}

	/**
	 * Creates a title bar for the pane which includes the view icon and title to the far left
	 */
	protected void createTitleBar() {
		// Only do this once.
		if (titleLabel == null) {
			// Title.
			titleLabel = new CLabel(viewForm, SWT.SHADOW_NONE);
			titleLabel.setAlignment(SWT.LEFT);
			titleLabel.setBackground(null, null);
			updateTitles();
			viewForm.setTopLeft(titleLabel);
			titleLabel.addMouseListener(mouseListener);
		}
		actionBar = new ToolBar(viewForm, SWT.FLAT | SWT.WRAP);
		/* collapse all */
		setCollapseAllButton();        
		/* expand all */
		setExpandAllButton();
		/* search */
		setSearchElementButton();
		/* graphically view */
		setGraphicalViewButton();
		viewForm.setTopCenter(actionBar);
	}

	/* 
	 * This is the method that converts the metamodel/profile before it is represented in the user interface.
	 * 
	 * (non-Javadoc)
	 * @see org.eclipse.weaver.editors.IWovenModelPanel#setInput(org.eclipse.emf.ecore.resource.ResourceSet)
	 */
	public void setInput(Object input) {
		((TreeViewer) viewer).setLabelProvider(new ByADLWovenModelLabelProvider(this.adapterFactory));
		if(input instanceof Resource) {
			// if the resource is a UML metamodel then it must be converted to an ecore metamodel
			if(this.isUML(input)) {
				this.transformer.convertUmlInput((Resource) input);
				this.ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_graphic_profile"));
			} else {
				this.ti.setToolTipText(WeaverByADLExtensionPlugin.getPlugin().getString("_UI_panel_action_graphic_metamodel"));        	
			}
			// set the resource of the viewer as the metamodel/profile passed in input
			this.viewer.setInput(input);
			// filters out all the elements of the woven metamodel
			// that are not a class, a structural feature or a package 
			DUALLyViewerFilter filter = new DUALLyViewerFilter();
			try {
				((SelectionTreeViewer) this.viewer).addFilter(filter);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
		((TreeViewer) viewer).expandToLevel(1);      
	}

	/**
	 * @param input the Resource 
	 * @return true if <code>input</code> is a UML-based Resource 
	 */
	private boolean isUML(Object input) {
		return input instanceof UMLResourceImpl
		&& ((UMLResourceImpl)input).getURI().fileExtension().equals("uml");
	}

	/**
	 * Adds an identifier adapter for every object in a resource,
	 * generates and ID for it,
	 * copy new resource in a another location if user want.
	 * 
	 * @param resource
	 * @param xmlFile
	 */
	@SuppressWarnings("deprecation")
	protected Resource addIdentifierAdapters(Resource resource, WeaverXMLMetadata xmlFile, String modelID){
		// if the elements of the resource contain xmi:id references they must be handled
		// as XMI elements.
		this.metadataFile = xmlFile;
		if(this.isXMI(resource)) {
			return this.addXMIIdentifierAdapters(resource, xmlFile, modelID);
		}
		if (resource instanceof XMIResource){
			XMIResource xmiRes = (XMIResource)resource;
			boolean flag = false;
			int a=0;
			//adds an identifier adapter for the resource as well
			xmiRes.eAdapters().add(new ResourceIdentifierAdapter(xmiRes));
			for (Iterator elements = resource.getAllContents();elements.hasNext(); ){
				Object obj = elements.next();
				if (obj instanceof EObject){
					//the adapter of the woven objects can be changed here
					IIdentifierAdapter adapter = new GenericIdentifierAdapter((EObject)obj);
					((EObject)obj).eAdapters().add(adapter);
					if (adapter.getID() == null){
						String className = (String)((EObject)obj).eClass().getName(); 
						do {//prevents from having the same ID 
							a ++;
						} while (xmiRes.getEObject(className + a)!= null);    				    				
						adapter.setID(className + a);		
						flag = true;
					}
				}
			}		
			try {
				if(flag){
					String filename = xmiRes.getURI().path();
					boolean response = MessageDialog.openQuestion(
							this.getControl().getShell(),
							WeaverByADLExtensionPlugin.getPlugin().getString("_UI_woven_panel_question_title"),
							WeaverByADLExtensionPlugin.getPlugin().getString("_UI_woven_panel_question_message",titleLabel.getText(),filename));
					if(!response){
						SaveAsDialog saveAsDialog= new SaveAsDialog(this.getControl().getShell());		
						saveAsDialog.open();
						IPath path= saveAsDialog.getResult();
						if (path != null) {
							URI uri = URI.createPlatformResourceURI(path.toString());
							xmiRes.setURI(uri);
							List modelFile = xmlFile.getWovenModels();
							List tempList = new ArrayList();
							for(int i=0;i<modelFile.size();i++){
								WovenModelDescriptor desc = (WovenModelDescriptor)modelFile.get(i);
								if(desc.getReferenceName().equals(this.getTitleLabel().getText())){
									if(desc.getModelFile().equals("null"))
										desc.setMetamodelFile(path.toString());
									else
										desc.setModelFile(path.toString());
								}
								tempList.add(desc);
							}
							try {
								// Config file
								File propertiesFile = xmlFile.getFile();
								WeaverXMLMetadata xmlFileTemp = new WeaverXMLMetadata(propertiesFile);
								xmlFileTemp.createXMLConfigFile(tempList,
										xmlFile.getWeavingModelPR(),
										xmlFile.getWeavingModelPanel(),
										xmlFile.getWModelName(),
										xmlFile.getMetamodelExtension(),
										xmlFile.getLocalMetamodelExtension(),
										xmlFile.getWeavingEcore());
							} catch (Exception e1) {
								e1.printStackTrace();
								MessageDialog.openWarning(
										this.getControl().getShell(),
										WeaverByADLExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_title"),
										WeaverByADLExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_message",e1.getMessage()));
							}
						}	
					}
					xmiRes.save(null);
					return xmiRes;
				}

			} catch (IOException e) {
				e.printStackTrace();
				MessageDialog.openWarning(
						this.getControl().getShell(),
						WeaverByADLExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_title"),
						WeaverByADLExtensionPlugin.getPlugin().getString("_UI_woven_panel_warning_message",e.getMessage()));
			}			
		}
		return resource;
	}

	/**
	 * @param resource the resource
	 * @return true if <code>resource</code> is an XMI resource
	 */
	private boolean isXMI(Resource resource) {
		if (resource instanceof XMIResource){
			Iterator elements = resource.getAllContents();
			Object obj = elements.next();
			if (obj instanceof EObject){
				IIdentifierAdapter adapter = new GenericIdentifierAdapter((EObject)obj);
				return !adapter.getID().toString().startsWith("/");
			}
		}
		return false;
	}

	/**
	 * adds an identifier adapter for every object in a resource,
	 * generates and ID for it.
	 * @param resource
	 * @param xmlFile
	 */
	protected Resource addXMIIdentifierAdapters(Resource resource, WeaverXMLMetadata xmlFile, String modelID){
		if (resource instanceof XMIResource){
			XMIResource xmiRes = (XMIResource)resource;
			int a=0;
			//adds an identifier adapter for the resource as well
			xmiRes.eAdapters().add(new ResourceIdentifierAdapter(xmiRes));
			for (Iterator elements = resource.getAllContents();elements.hasNext(); ){
				Object obj = elements.next();
				if (obj instanceof EObject){
					IIdentifierAdapter adapter = new XMIIdentifierAdapter((EObject)obj);
					((EObject)obj).eAdapters().add(adapter);
					if (adapter.getID() != null){
						String className = (String)((EObject)obj).eClass().getName(); 
						do {//prevents from having the same ID 
							a++;
						} while (xmiRes.getEObject(className + a)!= null);    				    				
						adapter.setID(className + a);		
					}
				}
			}		
		}
		return resource;
	}

	@Override
	public void dispose() {
		//((Resource) this.viewer.getInput()).unload();
		super.dispose();
	}
}
