package org.eclipse.weaver.extension.byADL.plugin;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;
import java.text.MessageFormat;
import java.util.HashMap;
import java.util.Map;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.Platform;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.common.util.WrappedException;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.osgi.framework.BundleContext;

/**
 * The main plug-in class to be used in the desktop.
 * 
 * @author - Ivano Malavolta (Department of Computer Science)<br>
 * 		   - Universit dell'Aquila (Italy)
 * @since 1.0
 */
public class WeaverByADLExtensionPlugin extends AbstractUIPlugin {
	//The shared instance.
	private static WeaverByADLExtensionPlugin plugin;
	//Resource bundle.
	private ResourceBundle resourceBundle;
	// the base URL of the plug-in
	protected URL baseURL;
	// the images contained inside the bundle
	public Map images = new HashMap();	
	
	/**
	 * Get the singleton instance.
	 */
	public static WeaverByADLExtensionPlugin getPlugin(){
		if(plugin == null)
			new WeaverByADLExtensionPlugin();
		return plugin;
	}		
	
	/**
	 * The constructor.
	 */
	public WeaverByADLExtensionPlugin() {
		super();
		plugin = this;
		try {
			resourceBundle = ResourceBundle.getBundle("org.eclipse.weaver.extension.byADL.WeaverByADLExtensionPluginResources");
		} catch (MissingResourceException x) {
			resourceBundle = null;
		}
	}

	/**
	 * This method is called upon plug-in activation
	 */
	public void start(BundleContext context) throws Exception {
		super.start(context);
	}

	/**
	 * This method is called when the plug-in is stopped
	 */
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
		images.clear();
		images = null;
	}

	/**
	 * Returns the shared instance.
	 */
	public static WeaverByADLExtensionPlugin getDefault() {
		return plugin;
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		ResourceBundle bundle = WeaverByADLExtensionPlugin.getDefault().getResourceBundle();
		try {
			return (bundle != null) ? bundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	/**
	 * Returns the plugin's resource bundle
	 */
	public ResourceBundle getResourceBundle() {
		return resourceBundle;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.util.ResourceLocator#getBaseURL()
	 */
	public URL getBaseURL() {
	    if (baseURL == null){
	    	try{
	    		
	    		URL pluginPropertiesURL = getClass().getResource("plugin.properties");
	    		if (pluginPropertiesURL == null){
		            String className = getClass().getName();
		            int index = className.lastIndexOf(".");
		            String resourceName = index == -1 ? "plugin.properties" : 
		            	className.substring(0, index + 1).replace('.','/') + "plugin.properties";
		            throw new MissingResourceException("Missing properties: " + resourceName, getClass().getName(), "plugin.properties");
	    		}
	    		else{
	          		baseURL = new URL (URI.createURI (pluginPropertiesURL.toString()).trimSegments(1).toString() + "/");
	          	}
	        }catch (IOException exception){
	        	throw new WrappedException(exception);
	        }
	    }
	    return baseURL;
	}

	/* (non-Javadoc)
	 * @see org.eclipse.emf.common.util.ResourceLocator#getImage(java.lang.String)
	 */
	public Object getImage(String key) {
		Object result = (URL)images.get(key);
	    if (result == null){
	        try{
	        	result = doGetImage(key);
	        }catch (MalformedURLException exception){
	        	throw new WrappedException(exception);
	        }catch (IOException exception){
	        	result = null;
	        }
	        images.put(key, result);
	    }
	    return result;
	}
	
	/**
	 * Does the work of fetching the image associated with the key.
	 * It ensures that the image exists.
	 * @param key the key of the image to fetch.
	 * @exception IOException if an image doesn't exist.
	 * @return the description of the image associated with the key.
	 */
	protected Object doGetImage(String key) throws IOException{
		URL url = new URL(getBundle().getEntry("/"),key);
		InputStream inputStream = url.openStream(); 
		inputStream.close();
		return url;
	}	

	public String getString(String key){
		return Platform.getResourceBundle(getBundle()).getString(key);
	}
	
	public String getString(String key, Object s1){
		return MessageFormat.format(getString(key), new Object [] { s1 });
	}
	
	public String getString(String key, Object s1, Object s2){
		return MessageFormat.format(getString(key), new Object [] { s1, s2 });
	}
	
	public String getString(String key, Object[] array){
		return MessageFormat.format(getString(key), array);
	}	
}
