package org.eclipse.weaver.extension.byADL.providers;

import java.util.Collection;

import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EClass;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.edit.EMFEditPlugin;
import org.eclipse.emf.edit.provider.IEditingDomainItemProvider;
import org.eclipse.emf.edit.provider.IItemLabelProvider;
import org.eclipse.emf.edit.provider.IItemPropertySource;
import org.eclipse.emf.edit.provider.IStructuredItemContentProvider;
import org.eclipse.emf.edit.provider.ITreeItemContentProvider;
import org.eclipse.weaver.editors.interfaces.IWeaverItemProvider;
import org.eclipse.weaver.extension.byADL.plugin.WeaverByADLExtensionPlugin;
import org.eclipse.weaver.extension.providers.*;


/**
 * WElementItemProvider byADL extension
 * 
 * @author - Ivano Malavolta (Department of Computer Science)<br>
 * 		   - Universit dell'Aquila (Italy)
 * @since 1.0
 */
public class ByADLWElementItemProvider extends WElementItemProvider 
	implements      
		IEditingDomainItemProvider,
		IStructuredItemContentProvider,
		ITreeItemContentProvider,
		IItemLabelProvider,
		IItemPropertySource, 
		IWeaverItemProvider
	{

    /**
     * This creates the extension item provider of byADL
     */
    public ByADLWElementItemProvider(){
        super();
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.edit.provider.ItemProviderAdapter#getImage(java.lang.Object)
     */
    public Object getImage(Object object){
    	EObject eObject = (EObject)object;
    	EClass eClass = eObject.eClass();
    	String name = eClass.getName();
    	String icon = getIconPath(name);
    	if (icon != null){
    		return URI.createURI(WeaverByADLExtensionPlugin.getPlugin().getImage(icon).toString() + "#" + eClass.getName());
    	}
    	else
    		return URI.createURI(getResourceLocator().getImage("full/obj16/Item").toString() + "#" + eClass.getName());
    }
    
    /* (non-Javadoc)
     * @see org.eclipse.emf.edit.command.CreateChildCommand$Helper#getCreateChildImage(java.lang.Object, java.lang.Object, java.lang.Object, java.util.Collection)
     */
    public Object getCreateChildImage(Object owner, Object feature, Object child, Collection selection){ 
    	EObject eObject = (EObject)child;
    	EClass eClass = eObject.eClass();
    	String name = eClass.getName();
    	String icon = getIconPath(name);    	
    	
    	if (icon != null){
    		return URI.createURI(WeaverByADLExtensionPlugin.getPlugin().getImage(icon).toString());
    	}
    	else
    		return EMFEditPlugin.INSTANCE.getImage("full/ctool16/CreateChild");
    }  
    
    /* 
     * This method overrides org.eclipse.weaver.extension.providers.WElementItemProvider.notifyChanged
     * because it generated an exception that is caught here.
     * 
     * (non-Javadoc)
     * @see org.eclipse.emf.common.notify.Adapter#notifyChanged(org.eclipse.emf.common.notify.Notification)
     */
    public void notifyChanged (Notification notification){    	
    	try {
			super.notifyChanged(notification);
		} catch (Exception e) {}
    }
}



